@@ -30,6 +30,9 @@ void error_internal( ERROR_CODE errcode, const char *file, int line );
 #define ERROR_OUTMEM()    error_internal( ERROR_CODE_OUTMEM, __FILE__, __LINE__ )
 #define ERROR_UNEXPECTED()    error_internal( ERROR_CODE_UNEXPECTED, __FILE__, __LINE__ )
 
+int is_compat;
+int files_count;
+
 /* strarray.c */
 typedef sl4_array_t strarray_t;
 strarray_t *strarray_new(  );
@@ -113,21 +116,21 @@ sl4_string_t *sl4_file_get_mimetype( sl4_file_t *file,
                                      sl4_string_t **suffix );
 
 /* tmpl_bool.c */
-sl4_string_t *tmpl_bool_get( const char *text, int is_required );
+sl4_string_t *tmpl_bool_get( const char *text, int is_required, char *item_xml );
 
 /* tmpl_integer.c */
-sl4_string_t *tmpl_integer_get( const char *text, int is_required );
+sl4_string_t *tmpl_integer_get( const char *text, int is_required, char *item_xml );
 
 /* tmpl_string.c */
-sl4_string_t *tmpl_string_get( const char *text, int is_required );
+sl4_string_t *tmpl_string_get( const char *text, int is_required, char *item_xml );
 
 /* tmpl_select.c */
 sl4_string_t *tmpl_select_get( const char *text, strarray_t * options,
-                               int is_required );
+                               int is_required, char *item_xml );
 
 /* tmpl_array.c */
 sl4_string_t *tmpl_array_get( const char *text, strarray_t * options,
-                              int is_required );
+                              int is_required, char *item_xml );
 
 /* tmpl_dataset.c */
 strarray_t *tmpl_dataset_get( const char *text, strarray_t * options,
@@ -141,16 +144,17 @@ sl4_string_t *tmpl_file_get( const char *text, strarray_t * options,
                              sl4_string_t **original_file_name,
                              sl4_string_t **mime_type,
                              sl4_string_t **file_size,
-                             sl4_string_t **suffux, int is_required );
+                             sl4_string_t **suffux, int is_required,
+                             sl4_string_t *tmpdir, char *item_xml );
 
 /* tmpl_rights.c */
 sl4_string_t *tmpl_rights_get( const char *text, sl4_string_t **use_cc,
                                sl4_string_t **cc_commercial_use,
                                sl4_string_t **cc_modification,
-                               int is_required );
+                               int is_required, char *item_xml );
 
 /* tmpl_indexes.c */
-sl4_string_t *tmpl_indexes_get( const char *text, int is_required );
+sl4_string_t *tmpl_indexes_get( const char *text, int is_required, char *item_xml );
 
 /* template.c */
 typedef struct _template_t {
