@@ -98,8 +98,7 @@ inifile_t *inifile_load( const char *inidir, const char *itemtype,
         m[4] = strarray_get( matches, 4 );      /* comment */
         ini->col[ini->ncol] =
           _inifile_column_new( m[1], m[2], m[3],
-                               sl4_string_get( inifile_path ), line,
-                               errmes );
+                               sl4_string_get( inifile_path ), line, errmes );
         if ( ini->col[ini->ncol] == NULL ) {
           sl4_string_delete( tmp );
           sl4_file_close( inifile );
@@ -161,8 +160,7 @@ static inifile_column_t *_inifile_column_new( sl4_string_t *fields,
                                               sl4_string_t *type,
                                               sl4_string_t *optional,
                                               const char *itemtype,
-                                              int line,
-                                              sl4_string_t *errmes )
+                                              int line, sl4_string_t *errmes )
 {
   inifile_column_t *col;
   sl4_string_t *tmp;
@@ -235,8 +233,7 @@ static inifile_column_t *_inifile_column_new( sl4_string_t *fields,
   } else {
     strarray_t *matches = strarray_new(  );
     if ( myonig_match
-         ( "/select\\(\\s*(.+)\\s*\\)/", sl4_string_get( type ),
-           matches ) ) {
+         ( "/select\\(\\s*(.+)\\s*\\)/", sl4_string_get( type ), matches ) ) {
       sl4_string_t *m = strarray_get( matches, 1 );
       if ( m == NULL ) {
         ERROR_UNEXPECTED(  );
@@ -256,27 +253,69 @@ static inifile_column_t *_inifile_column_new( sl4_string_t *fields,
       }
     } else
       if ( myonig_match
-           ( "/array\\(\\s*(.+)\\s*\\)/", sl4_string_get( type ),
-             matches ) ) {
-      sl4_string_t *m = strarray_get( matches, 1 );
-      if ( m == NULL ) {
+           ( "/array\\(\\s*([^,]+)\\s*(,\\s*'(.+)')?\\s*\\)/",
+             sl4_string_get( type ), matches ) ) {
+      sl4_string_t *tag, *sep;
+      tag = strarray_get( matches, 1 );
+      if ( tag == NULL ) {
         ERROR_UNEXPECTED(  );
       }
-      sl4_string_trim( m );
+      sep = strarray_get( matches, 3 );
       col->type = INIFILE_TYPE_ARRAY;
       col->options = strarray_new(  );
       if ( col->options == NULL ) {
         ERROR_OUTMEM(  );
       }
-      tmp = sl4_string_new( sl4_string_get( m ) );
+      sl4_string_trim( tag );
+      tmp = sl4_string_new( sl4_string_get( tag ) );
       if ( tmp == NULL ) {
         ERROR_OUTMEM(  );
       }
       strarray_add( col->options, tmp );
+      if ( sep == NULL ) {
+        tmp = sl4_string_new( "\\n" );
+      } else {
+        tmp = sl4_string_new( sl4_string_get( sep ) );
+      }
+      if ( tmp == NULL ) {
+        ERROR_OUTMEM(  );
+      }
+      sl4_string_convert_format( tmp );
+      strarray_add( col->options, tmp );
     } else
       if ( myonig_match
-           ( "/file\\(\\s*(.+)\\s*\\)/", sl4_string_get( type ),
-             matches ) ) {
+           ( "/dataset\\(\\s*([^,]+)\\s*(,\\s*'(.+)')?\\s*\\)/",
+             sl4_string_get( type ), matches ) ) {
+      sl4_string_t *tag, *sep;
+      tag = strarray_get( matches, 1 );
+      if ( tag == NULL ) {
+        ERROR_UNEXPECTED(  );
+      }
+      sep = strarray_get( matches, 3 );
+      col->type = INIFILE_TYPE_DATASET;
+      col->options = strarray_new(  );
+      if ( col->options == NULL ) {
+        ERROR_OUTMEM(  );
+      }
+      sl4_string_trim( tag );
+      tmp = sl4_string_new( sl4_string_get( tag ) );
+      if ( tmp == NULL ) {
+        ERROR_OUTMEM(  );
+      }
+      strarray_add( col->options, tmp );
+      if ( sep == NULL ) {
+        tmp = sl4_string_new( "\\n" );
+      } else {
+        tmp = sl4_string_new( sl4_string_get( sep ) );
+      }
+      if ( tmp == NULL ) {
+        ERROR_OUTMEM(  );
+      }
+      sl4_string_convert_format( tmp );
+      strarray_add( col->options, tmp );
+    } else
+      if ( myonig_match
+           ( "/file\\(\\s*(.+)\\s*\\)/", sl4_string_get( type ), matches ) ) {
       sl4_string_t *m = strarray_get( matches, 1 );
       if ( m == NULL ) {
         ERROR_UNEXPECTED(  );
