@@ -25,6 +25,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <getopt.h>
+#include <unistd.h>
+#include <limits.h>
 
 #include "cole.h"
 #include "libsl4.h"
@@ -41,29 +44,52 @@ static int proc_sheet( cexcel_sheet_t * sheet, sl4_string_t *tmpdir,
                        int *item_id, const char *bindir,
                        sl4_string_t *errmes, int is_index );
 
+struct option longopts[] = {
+  {"index",      no_argument,       0,  'i' },
+  {"compat34",   no_argument,       0,  'c' },
+  {"help",       no_argument,       0,  'h' },
+  {0,            0,                 0,  0 }
+};
+
 int main( int argc, char *argv[] )
 {
   int ret, is_index;
+  int long_index, opt;
   const char *xls_file, *zip_file;
   is_index = 0;
-  if ( argc == 4 ) {
-    if ( strcmp( argv[1], "-i" ) == 0 || strcmp( argv[1], "--index" ) == 0 ) {
-      is_index = 1;
-    } else {
-      print_usage( argv[0] );
-      return 1;
+  is_compat = 0;
+  files_count = 1;
+
+  long_index=0;
+  while ((opt = getopt_long(argc, argv,"ich",
+          longopts, &long_index )) != -1) {
+    switch (opt) {
+      case 'i' :
+        is_index = 1;
+        break;
+      case 'c' :
+        is_compat = 1;
+        break;
+      case 'h' :
+      default:
+        print_usage( argv[0] );
+        return 1;
     }
-  } else if ( argc != 3 ) {
+  }
+
+  if ( (optind + 2) == argc) {
+    xls_file = argv[optind];
+    zip_file = argv[optind + 1];
+  } else {
     print_usage( argv[0] );
     return 1;
   }
-  if ( is_index ) {
-    xls_file = argv[2];
-    zip_file = argv[3];
-  } else {
-    xls_file = argv[1];
-    zip_file = argv[2];
+
+  if (!is_compat && is_index) {
+	  printf("%s don't suport index option for Xoonips 4.0\n", argv[0]);
+	  return 0;
   }
+
   ret = proc_main( argv[0], xls_file, zip_file, is_index );
   if ( ret != 0 ) {
     return 1;
@@ -77,6 +103,7 @@ static void print_usage( const char *bin )
   printf( "\n" );
   printf( "options:\n" );
   printf( "  -i or --index : convert index list (experimental)\n" );
+  printf( "  -c or --compat34 : use xoonips v3.4 format\n" );
   printf( "  -h or --help  : print this message\n" );
 }
 
@@ -115,7 +142,9 @@ static int proc_main( const char *bin, const char *xls_file,
   }
 
   tmpdir = tempdir_new(  );
-  tempdir_mkdir( tmpdir, "/files" );
+  if (is_compat) {
+    tempdir_mkdir( tmpdir, "/files" );
+  }
 
   for ( i = 1; i <= nums; i++ ) {
     sheet = cexcel_book_get_sheet( book, i );
@@ -158,8 +187,10 @@ static int proc_sheet( cexcel_sheet_t * sheet, sl4_string_t *tmpdir,
   int err, count;
   template_t *tmpl;
   inifile_t *ini;
-  const char *sname, *cname, *config_path;
+  const char *sname, *cname;
   unsigned int rows, cols, i;
+  sl4_string_t *tmpdir_row;
+  char config_path[PATH_MAX], tmp_str1[PATH_MAX];
 #ifdef WIN32
   char buf[_MAX_PATH];
   char *etc = "/etc";
@@ -168,11 +199,17 @@ static int proc_sheet( cexcel_sheet_t * sheet, sl4_string_t *tmpdir,
   }
   strcpy( buf, bindir );
   strcat( buf, etc );
-  config_path = buf;
+  strcpy(config_path, buf);
 #else
-  config_path = E2X_CONFIG_PATH;
+  strcpy(config_path, E2X_CONFIG_PATH);
 #endif
 
+  if (is_compat) {
+	  strcat(config_path, "/3.4");
+  } else {
+	  strcat(config_path, "/4.0");
+  }
+
   err = XLS2XNI_ERROR_SUCCESS;
 
   sname = cexcel_sheet_get_name( sheet );
@@ -199,15 +236,45 @@ static int proc_sheet( cexcel_sheet_t * sheet, sl4_string_t *tmpdir,
     template_delete( tmpl );
     return -1;
   }
+
+  tmpdir_row = sl4_string_new( NULL );
   for ( i = 1; i < rows; i++ ) {
-    if ( template_assign
-         ( tmpl, sheet, ini, tmpdir, i, *item_id, errmes, is_index )
-         != 0 ) {
+    if (!is_compat){
+        // make a dir to a row of sheet, name as item_id
+        sprintf(tmp_str1, "/%d", *item_id);
+        tempdir_mkdir( tmpdir, tmp_str1 );
+        sl4_string_set(tmpdir_row, sl4_string_get(tmpdir));
+        sl4_string_append(tmpdir_row, tmp_str1);
+        err = template_assign
+               ( tmpl, sheet, ini, tmpdir_row, i, *item_id, errmes, is_index);
+    } else {
+        err = template_assign
+               ( tmpl, sheet, ini, tmpdir, i, *item_id, errmes, is_index);
+    }
+
+    if ( err != 0 ) {
       err = -1;
       break;
     }
+
+    if (!is_compat) {
+      sprintf(tmp_str1, "%s.zip", sl4_string_get(tmpdir_row));
+      err = minizip_create( tmp_str1, sl4_string_get(tmpdir_row) );
+      switch ( err ) {
+      case -1:
+        printf( "ERROR: Failed to create XooNIps Import File \"%s\"\n", tmp_str1 );
+        break;
+      case -2:
+        printf( "ERROR: No Items Found in item_id %d temporary dicrectory\n", *item_id );
+        break;
+      default:
+        break;
+      }
+      directory_remove(sl4_string_get(tmpdir_row));
+    }
     ( *item_id )++;
   }
+  sl4_string_delete(tmpdir_row);
   if ( is_index ) {
     if ( template_merge_rowset( tmpl, tmpdir, count, *item_id, "index" ) !=
          0 ) {
