@@ -99,16 +99,14 @@ int minizip_create( const char *filename, const char *rootdir )
       ERROR_OUTMEM(  );
     }
     sl4_string_append( realpath_inzip, "/" );
-    sl4_string_append( realpath_inzip,
-                       sl4_string_get( virtualpath_inzip ) );
+    sl4_string_append( realpath_inzip, sl4_string_get( virtualpath_inzip ) );
 
     zi.tmz_date.tm_sec = zi.tmz_date.tm_min = zi.tmz_date.tm_hour = 0;
     zi.tmz_date.tm_mday = zi.tmz_date.tm_mon = zi.tmz_date.tm_year = 0;
     zi.dosDate = 0;
     zi.internal_fa = 0;
     zi.external_fa = 0;
-    filetime( sl4_string_get( realpath_inzip ), &zi.tmz_date,
-              &zi.dosDate );
+    filetime( sl4_string_get( realpath_inzip ), &zi.tmz_date, &zi.dosDate );
 
     /* open file in zip */
     err =
@@ -168,7 +166,6 @@ int minizip_create( const char *filename, const char *rootdir )
   return 0;
 }
 
-
 static uLong filetime( const char *f, tm_zip * tmzip, uLong * dt )
 {
 #ifdef WIN32
