@@ -114,6 +114,9 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
   const char *address, *text, *sname;
   sl4_hash_table_t *rep_hash, *dataset_hash;
   inifile_column_t *inicol;
+  char tmp_str1[128];
+
+  char *item_xml;
 
   sname = cexcel_sheet_get_name( sheet );
 
@@ -153,7 +156,13 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
   /* system.itemtype */
   {
     sl4_string_t *itemtype_str;
-    itemtype_str = sl4_string_new( sname );
+    if ( !is_compat ) {
+      strcpy(tmp_str1, sname);
+      tmp_str1[0] = toupper(tmp_str1[0]);
+      itemtype_str = sl4_string_new( tmp_str1 );
+    } else {
+      itemtype_str = sl4_string_new( sname );
+    }
     if ( itemtype_str == NULL ) {
       ERROR_OUTMEM(  );
     }
@@ -177,11 +186,19 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
         return -1;
       }
     }
+    if ( !is_compat ) {
+        item_xml = strchr(sl4_string_get(strarray_get(inicol->field, 0)), '.');
+        if (item_xml == NULL) {
+            item_xml = sl4_string_get(strarray_get(inicol->field, 0));
+        } else {
+            item_xml++;
+        }
+    }
     switch ( inicol->type ) {
     case INIFILE_TYPE_BOOL:
       {
         sl4_string_t *bool_str;
-        bool_str = tmpl_bool_get( text, inicol->is_required );
+        bool_str = tmpl_bool_get( text, inicol->is_required, item_xml );
         if ( bool_str == NULL ) {
           sl4_string_sprintf( errmes,
                               "Invalid Cell(%s) Found in Sheet \"%s\". Data Type is Not Boolean.",
@@ -195,7 +212,7 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
     case INIFILE_TYPE_INTEGER:
       {
         sl4_string_t *integer_str;
-        integer_str = tmpl_integer_get( text, inicol->is_required );
+        integer_str = tmpl_integer_get( text, inicol->is_required, item_xml );
         if ( integer_str == NULL ) {
           sl4_string_sprintf( errmes,
                               "Invalid Cell(%s) Found in Sheet \"%s\". Data Type is Not Integer.",
@@ -210,15 +227,21 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
     case INIFILE_TYPE_STRING:
       {
         sl4_string_t *string_str;
-        string_str = tmpl_string_get( text, inicol->is_required );
+        string_str = tmpl_string_get( text, inicol->is_required, item_xml );
         if ( string_str == NULL ) {
           sl4_string_sprintf( errmes,
                               "Invalid Cell(%s) Found in Sheet \"%s\". Data is Not Selectable Name.",
                               address, sname );
           return -1;
         }
-        _template_assign_fields( rep_hash, inicol->field, string_str, 1,
-                                 NULL );
+        if ( is_compat ) {
+            _template_assign_fields( rep_hash, inicol->field, string_str, 1,
+                                     NULL );
+        } else {
+            _template_assign_fields( rep_hash, inicol->field, string_str, 0,
+                                     NULL );
+        }
+
         sl4_string_delete( string_str );
       }
       break;
@@ -232,21 +255,23 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
           tmpl_file_get( text, inicol->options, item_id, sname, address,
                          &file, &file_name_inzip, &original_file_name,
                          &mime_type, &file_size, &suffix,
-                         inicol->is_required );
+                         inicol->is_required, tmpdir, item_xml );
 
         if ( file_str == NULL ) {
           sl4_string_sprintf( errmes,
-                              "Invalid Cell(%s) Found in Sheet \"%s\". File Not Found.",
+                              "Invalid Cell(%s) Found in Sheet \"%s\". File Not Found or not allow duplicate.",
                               address, sname );
           return -1;
         }
 
         if ( sl4_string_empty( file_str ) == 0 ) {
-          /* copy file to temporary directory */
-          if ( tempdir_copyfile
-               ( tmpdir, sl4_file_get_path( file ),
-                 sl4_string_get( file_name_inzip ) ) != 0 ) {
-            ERROR_UNEXPECTED(  );
+          if ( is_compat ) {
+              /* copy file to temporary directory */
+              if ( tempdir_copyfile
+                   ( tmpdir, sl4_file_get_path( file ),
+                     sl4_string_get( file_name_inzip ) ) != 0 ) {
+                ERROR_UNEXPECTED(  );
+              }
           }
           sl4_string_delete( file_name_inzip );
           sl4_file_delete( file );
@@ -280,15 +305,20 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
       {
         sl4_string_t *select_str;
         select_str =
-          tmpl_select_get( text, inicol->options, inicol->is_required );
+          tmpl_select_get( text, inicol->options, inicol->is_required, item_xml );
         if ( select_str == NULL ) {
           sl4_string_sprintf( errmes,
                               "Invalid Cell(%s) Found in Sheet \"%s\". Data Type Mismatch.",
                               address, sname );
           return -1;
         }
-        _template_assign_fields( rep_hash, inicol->field, select_str, 1,
-                                 NULL );
+        if ( is_compat ) {
+            _template_assign_fields( rep_hash, inicol->field, select_str, 1,
+                                     NULL );
+        } else {
+            _template_assign_fields( rep_hash, inicol->field, select_str, 0,
+                                     NULL );
+        }
         sl4_string_delete( select_str );
       }
       break;
@@ -296,7 +326,7 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
       {
         sl4_string_t *array_str;
         array_str =
-          tmpl_array_get( text, inicol->options, inicol->is_required );
+          tmpl_array_get( text, inicol->options, inicol->is_required, item_xml );
         if ( array_str == NULL ) {
           sl4_string_sprintf( errmes,
                               "Invalid Cell(%s) Found in Sheet \"%s\". Data Type Mismatch.",
@@ -377,14 +407,18 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
         sl4_string_t *rights, *use_cc, *cc_commercial_use, *cc_modification;
         rights =
           tmpl_rights_get( text, &use_cc, &cc_commercial_use,
-                           &cc_modification, inicol->is_required );
+                           &cc_modification, inicol->is_required, item_xml );
         if ( rights == NULL ) {
           sl4_string_sprintf( errmes,
                               "Invalid Cell(%s) Found in Sheet \"%s\". Data Type Mismatch.",
                               address, sname );
           return -1;
         }
-        _template_assign_fields( rep_hash, inicol->field, rights, 1, NULL );
+        if ( is_compat ) {
+          _template_assign_fields( rep_hash, inicol->field, rights, 1, NULL );
+        } else {
+          _template_assign_fields( rep_hash, inicol->field, rights, 0, NULL );
+        }
         _template_assign_fields( rep_hash, inicol->field, use_cc, 0,
                                  ".use_cc" );
         _template_assign_fields( rep_hash, inicol->field, cc_commercial_use,
@@ -400,7 +434,7 @@ int template_assign( template_t * tmpl, cexcel_sheet_t * sheet,
     case INIFILE_TYPE_INDEXES:
       {
         sl4_string_t *indexes_str;
-        indexes_str = tmpl_indexes_get( text, inicol->is_required );
+        indexes_str = tmpl_indexes_get( text, inicol->is_required, item_xml );
         if ( indexes_str == NULL ) {
           sl4_string_sprintf( errmes,
                               "Invalid Cell(%s) Found in Sheet \"%s\". Data Type Mismatch.",
