@@ -31,12 +31,20 @@
 #include "excel2xoonips.h"
 
 sl4_string_t *tmpl_array_get( const char *text, strarray_t * options,
-                              int is_required )
+                              int is_required, char *item_xml )
 {
   size_t ii;
   strarray_t *arr;
   sl4_string_t *tag, *sep, *str, *stag, *etag, *tmp;
   str = sl4_string_new( text );
+  if ( !is_compat && sl4_string_empty(str) ) {
+    if ( is_required ) {
+      sl4_string_delete( str );
+      return NULL;
+    } else {
+      return str;
+    }
+  }
   tag = strarray_get( options, 0 );
   sep = strarray_get( options, 1 );
   arr = strarray_explode( sep, str );
@@ -58,9 +66,26 @@ sl4_string_t *tmpl_array_get( const char *text, strarray_t * options,
       sl4_string_delete( str );
       return NULL;
     }
-    sl4_string_append( str, sl4_string_get( stag ) );
-    sl4_string_append( str, sl4_string_get( tmp ) );
-    sl4_string_append( str, sl4_string_get( etag ) );
+    if (is_compat) {
+      sl4_string_append( str, sl4_string_get( stag ) );
+      sl4_string_append( str, sl4_string_get( tmp ) );
+      sl4_string_append( str, sl4_string_get( etag ) );
+    } else {
+      sl4_string_t *row_tmp = sl4_string_new( NULL );
+      if ( row_tmp == NULL ) {
+        ERROR_OUTMEM(  );
+      }
+      if (strcmp(item_xml, "keyword") == 0 ) { // *.keyword
+        sl4_string_sprintf( row_tmp, "<C:%s C:type=\"keyword\" C:keyword_id=\"%d\">%s</C:%s>", item_xml, ii + 1, sl4_string_get(tmp), item_xml);
+      } else {
+        sl4_string_sprintf( row_tmp, "<C:%s C:type=\"extend\" C:occurrence_number=\"%d\">%s</C:%s>", item_xml, ii + 1, sl4_string_get(tmp), item_xml);
+      }
+      if ( ii > 0 ) {
+    	sl4_string_append( str, "\n" );
+      }
+      sl4_string_append( str, sl4_string_get( row_tmp ) );
+      sl4_string_delete( row_tmp );
+    }
   }
   sl4_string_delete( etag );
   sl4_string_delete( stag );
