@@ -107,7 +107,7 @@ int sl4_string_htmlspecialchars( sl4_string_t *str )
       pos += 5;
       break;
     case '\'':
-      if ( sl4_string_replace( str, pos, 1, "&039;" ) != 0 ) {
+      if ( sl4_string_replace( str, pos, 1, "&#039;" ) != 0 ) {
         ERROR_OUTMEM(  );
       }
       pos += 4;
