@@ -25,7 +25,7 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-/* $Id: file.c,v 1.1.1.1 2007/10/31 04:26:32 orrisroot Exp $ */
+/* $Id: file.c,v 1.2 2008/01/24 14:59:41 orrisroot Exp $ */
 
 #define LIBSL4_EXPORTS
 
@@ -109,8 +109,7 @@
 
 static sl4_file_t *_sl4_file_new_sub(  );
 static int _sl4_file_set_path( sl4_file_t *file, const char *path );
-static int _sl4_file_set_absolute_path( sl4_file_t *file,
-                                        const char *path );
+static int _sl4_file_set_absolute_path( sl4_file_t *file, const char *path );
 static char *_sl4_file_get_cwd_sub(  );
 static char *_sl4_file_get_parent_directory( const char *path );
 static unsigned char _sl4_file_check_attribute( const char *path );
@@ -227,7 +226,7 @@ LIBSL4_API off_t sl4_file_get_size( sl4_file_t *file )
   h = CreateFile( file->path, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
                   OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
   if ( h == INVALID_HANDLE_VALUE ) {
-    return ( off_t ) - 1;
+    return ( off_t ) -1;
   }
   low = GetFileSize( h, &high );
   ret = ( off_t ) high *( 2 ^ 32 ) + ( off_t ) low;
@@ -235,13 +234,13 @@ LIBSL4_API off_t sl4_file_get_size( sl4_file_t *file )
 #else
   if ( lstat( file->path, &sb ) != 0 ) {
     /* usually file not found */
-    return ( off_t ) - 1;
+    return ( off_t ) -1;
   }
   if ( S_ISREG( sb.st_mode ) ) {
     ret = sb.st_size;
   } else {
     /* file is not regular file */
-    return ( off_t ) - 1;
+    return ( off_t ) -1;
   }
 #endif
   return ret;
@@ -501,8 +500,7 @@ static int _sl4_file_set_path( sl4_file_t *file, const char *path )
   return _sl4_file_set_absolute_path( file, path );
 }
 
-static int _sl4_file_set_absolute_path( sl4_file_t *file,
-                                        const char *path )
+static int _sl4_file_set_absolute_path( sl4_file_t *file, const char *path )
 {
   size_t i, len;
   char *ret;
